document.addEventListener('DOMContentLoaded', function() {
  // Step elements
  const steps = [
    document.getElementById('step-1'),
    document.getElementById('step-2'),
    document.getElementById('step-3')
  ];
  
  // Step navigation elements
  const stepCircles = document.querySelectorAll('.step-circle');
  const stepLabels = document.querySelectorAll('.step-label');
  const stepProgress = document.querySelectorAll('.step-progress');
  
  // Buttons
  const nextStep1Button = document.getElementById('next-step-1');
  const prevStep2Button = document.getElementById('prev-step-2');
  const nextStep2Button = document.getElementById('next-step-2');
  const prevStep3Button = document.getElementById('prev-step-3');
  const paymentButton = document.getElementById('payment-button');
  
  // Video upload elements
  const videoUploadBox = document.getElementById('video-upload-box');
  const videoInput = document.getElementById('video');
  const videoPreview = document.getElementById('video-preview');
  
  // Form data
  let formData = {};
  
  // Current step
  let currentStep = 0;
  
  // Function to update the UI based on the current step
  function updateStepUI(step) {
    // Hide all steps
    steps.forEach(s => s.style.display = 'none');
    
    // Show current step
    steps[step].style.display = 'block';
    
    // Update step indicators
    stepCircles.forEach((circle, index) => {
      // Reset all circles
      circle.classList.remove('active', 'completed');
      
      if (index < step) {
        circle.classList.add('completed');
      } else if (index === step) {
        circle.classList.add('active');
      }
    });
    
    // Update step labels
    stepLabels.forEach((label, index) => {
      label.classList.remove('active');
      if (index === step) {
        label.classList.add('active');
      }
    });
    
    // Update progress indicators
    stepProgress.forEach((progress, index) => {
      progress.classList.remove('completed');
      if (index < step) {
        progress.classList.add('completed');
      }
    });
  }
  
  // Validate form data for each step
  function validateStep(step) {
    let isValid = true;
    const errors = {};
    
    if (step === 0) {
      // Personal Information validation
      const requiredFields = ['firstName', 'lastName', 'email', 'phone', 'age', 'gender', 'address', 'city', 'country', 'emergencyContactName', 'emergencyContactPhone'];
      
      requiredFields.forEach(field => {
        const input = document.getElementById(field);
        if (!input.value.trim()) {
          isValid = false;
          errors[field] = 'This field is required';
        }
      });
      
      // Email validation
      const email = document.getElementById('email').value;
      if (email && !/^\S+@\S+\.\S+$/.test(email)) {
        isValid = false;
        errors['email'] = 'Please enter a valid email address';
      }
      
      // Age validation
      const age = document.getElementById('age').value;
      if (age && parseInt(age) < 18) {
        isValid = false;
        errors['age'] = 'You must be at least 18 years old';
      }
    } else if (step === 1) {
      // Video submission validation
      const reason = document.getElementById('reason').value;
      if (!reason.trim()) {
        isValid = false;
        errors['reason'] = 'Please tell us why you should be chosen';
      }
      
      if (!formData.video) {
        isValid = false;
        errors['video'] = 'Please upload a video';
      }
    }
    
    // Display validation errors
    Object.keys(errors).forEach(field => {
      const errorElement = document.getElementById(`${field}-error`);
      if (errorElement) {
        errorElement.textContent = errors[field];
      }
    });
    
    return isValid;
  }
  
  // Clear previous error messages
  function clearErrors() {
    const errorElements = document.querySelectorAll('.error-message');
    errorElements.forEach(element => {
      element.textContent = '';
    });
  }
  
  // Event listeners for navigation buttons
  nextStep1Button.addEventListener('click', function() {
    clearErrors();
    if (validateStep(0)) {
      // Collect form data
      const formElements = document.getElementById('personal-info-form').elements;
      for (let i = 0; i < formElements.length; i++) {
        if (formElements[i].name) {
          formData[formElements[i].name] = formElements[i].value;
        }
      }
      
      currentStep = 1;
      updateStepUI(currentStep);
    }
  });
  
  prevStep2Button.addEventListener('click', function() {
    currentStep = 0;
    updateStepUI(currentStep);
  });
  
  nextStep2Button.addEventListener('click', function() {
    clearErrors();
    if (validateStep(1)) {
      // Collect form data
      const formElements = document.getElementById('video-submission-form').elements;
      for (let i = 0; i < formElements.length; i++) {
        if (formElements[i].name && formElements[i].name !== 'video') {
          formData[formElements[i].name] = formElements[i].value;
        }
      }
      
      currentStep = 2;
      updateStepUI(currentStep);
    }
  });
  
  prevStep3Button.addEventListener('click', function() {
    currentStep = 1;
    updateStepUI(currentStep);
  });
  
  // Video upload handling
  videoUploadBox.addEventListener('click', function() {
    videoInput.click();
  });
  
  videoInput.addEventListener('change', function(e) {
    if (e.target.files && e.target.files[0]) {
      const file = e.target.files[0];
      
      // Store the video file in formData
      formData.video = file;
      
      // Create object URL for preview
      const videoURL = URL.createObjectURL(file);
      videoPreview.src = videoURL;
      videoPreview.style.display = 'block';
      
      // Clear any previous error
      document.getElementById('video-error').textContent = '';
    }
  });
  
  // Handle drag and drop for video upload
  videoUploadBox.addEventListener('dragover', function(e) {
    e.preventDefault();
    e.stopPropagation();
    videoUploadBox.style.borderColor = '#00f3ff';
  });
  
  videoUploadBox.addEventListener('dragleave', function(e) {
    e.preventDefault();
    e.stopPropagation();
    videoUploadBox.style.borderColor = '#444';
  });
  
  videoUploadBox.addEventListener('drop', function(e) {
    e.preventDefault();
    e.stopPropagation();
    videoUploadBox.style.borderColor = '#444';
    
    if (e.dataTransfer.files && e.dataTransfer.files[0]) {
      const file = e.dataTransfer.files[0];
      
      // Check if it's a video file
      if (file.type.startsWith('video/')) {
        // Update the file input
        videoInput.files = e.dataTransfer.files;
        
        // Store the video file in formData
        formData.video = file;
        
        // Create object URL for preview
        const videoURL = URL.createObjectURL(file);
        videoPreview.src = videoURL;
        videoPreview.style.display = 'block';
        
        // Clear any previous error
        document.getElementById('video-error').textContent = '';
      } else {
        document.getElementById('video-error').textContent = 'Please upload a video file';
      }
    }
  });
  
  // Handle payment
  paymentButton.addEventListener('click', function() {
    // Simulate payment processing
    paymentButton.disabled = true;
    paymentButton.textContent = 'Processing...';
    
    // Simulate API call delay
    setTimeout(function() {
      document.getElementById('payment-success').style.display = 'block';
      paymentButton.style.display = 'none';
      
      // In a real application, you would send the form data to your server here
      console.log('Registration data:', formData);
      
      // Redirect to confirmation page after 3 seconds
      setTimeout(function() {
        alert('Registration successful! We will contact you soon.');
        window.location.href = 'index.html';
      }, 3000);
    }, 1500);
  });
  
  // Initialize the form
  updateStepUI(currentStep);
}); 