import React, { useState, useEffect } from 'react';
import styled, { keyframes } from 'styled-components';

const glow = keyframes`
  0% {
    box-shadow: 0 0 10px rgba(255, 0, 60, 0.5);
  }
  50% {
    box-shadow: 0 0 20px rgba(255, 0, 60, 0.8), 0 0 30px rgba(255, 0, 60, 0.5);
  }
  100% {
    box-shadow: 0 0 10px rgba(255, 0, 60, 0.5);
  }
`;

const float = keyframes`
  0% {
    transform: translateY(0px);
  }
  50% {
    transform: translateY(-10px);
  }
  100% {
    transform: translateY(0px);
  }
`;

const fadeInUp = keyframes`
  from {
    opacity: 0;
    transform: translateY(20px);
  }
  to {
    opacity: 1;
    transform: translateY(0);
  }
`;

const FeatureContainer = styled.div`
  width: 100%;
  max-width: 1000px;
  margin: 20px auto 40px;
  background: linear-gradient(to bottom, rgba(58, 43, 131, 0.7), rgba(42, 27, 99, 0.8));
  border-radius: 10px;
  border: 1px solid rgba(255, 51, 102, 0.4);
  overflow: hidden;
  animation: ${glow} 3s infinite;
  position: relative;
  z-index: 2;
  box-shadow: 0 10px 30px rgba(153, 102, 255, 0.3);
  
  &:before {
    content: '';
    position: absolute;
    top: 0;
    left: 0;
    right: 0;
    height: 3px;
    background: linear-gradient(to right, transparent, var(--neon-red), transparent);
  }
`;

const FeatureHeader = styled.div`
  padding: 20px;
  text-align: center;
  border-bottom: 1px solid rgba(255, 51, 102, 0.3);
`;

const FeatureTitle = styled.h2`
  color: white;
  font-size: 2rem;
  margin: 0;
  text-transform: uppercase;
  letter-spacing: 2px;
  animation: ${float} 3s ease-in-out infinite;
  
  @media (max-width: 768px) {
    font-size: 1.5rem;
  }
`;

const FeatureContent = styled.div`
  padding: 20px;
  display: flex;
  flex-direction: column;
  align-items: center;
`;

const EventList = styled.div`
  width: 100%;
  display: flex;
  flex-direction: column;
  gap: 15px;
`;

const EventItem = styled.div`
  display: flex;
  background: linear-gradient(135deg, rgba(58, 43, 131, 0.7) 0%, rgba(42, 27, 99, 0.7) 100%);
  border-radius: 8px;
  overflow: hidden;
  transition: transform 0.3s ease;
  animation: ${fadeInUp} 0.5s ease-out;
  animation-fill-mode: both;
  animation-delay: ${props => props.index * 0.1}s;
  border: 1px solid rgba(255, 255, 255, 0.1);
  
  &:hover {
    transform: scale(1.02);
    background: linear-gradient(135deg, rgba(58, 43, 131, 0.7) 0%, rgba(255, 51, 102, 0.3) 100%);
    border-color: rgba(255, 51, 102, 0.3);
  }
  
  @media (max-width: 768px) {
    flex-direction: column;
  }
`;

const EventDate = styled.div`
  background-color: var(--neon-red);
  color: white;
  padding: 20px;
  min-width: 100px;
  display: flex;
  flex-direction: column;
  align-items: center;
  justify-content: center;
  font-weight: bold;
  
  .month {
    font-size: 0.9rem;
    text-transform: uppercase;
  }
  
  .day {
    font-size: 1.8rem;
  }
  
  .year {
    font-size: 0.9rem;
  }
  
  @media (max-width: 768px) {
    padding: 10px;
    flex-direction: row;
    gap: 5px;
    
    .day {
      font-size: 1.2rem;
    }
  }
`;

const EventDetails = styled.div`
  padding: 15px;
  flex-grow: 1;
`;

const EventTitle = styled.h3`
  color: white;
  margin: 0 0 10px 0;
  font-size: 1.2rem;
`;

const EventDescription = styled.p`
  color: #ffffff;
  margin: 0;
  font-size: 0.9rem;
`;

const EventTag = styled.span`
  display: inline-block;
  background-color: rgba(255, 51, 102, 0.3);
  color: white;
  border-radius: 20px;
  padding: 3px 10px;
  font-size: 0.8rem;
  margin-top: 10px;
  margin-right: 5px;
`;

const upcomingEvents = [
  {
    id: 1,
    date: { month: 'Sep', day: 15, year: 2025 },
    title: 'Squid Game 2 Qualifying Round',
    description: 'First round of qualifications for the upcoming season. Limited spots available!',
    tags: ['Competition', 'Live Stream']
  },
  {
    id: 3,
    date: { month: 'Oct', day: 20, year: 2025 },
    title: 'Extreme Survival Challenge',
    description: 'A 48-hour survival challenge with a $1,000,000 prize for the last person standing.',
    tags: ['Endurance', 'Big Prize']
  }
];

const FeatureSection = () => {
  const [visibleEvents, setVisibleEvents] = useState([]);
  
  useEffect(() => {
    // Staggered animation for events
    upcomingEvents.forEach((event, index) => {
      setTimeout(() => {
        setVisibleEvents(prev => [...prev, event]);
      }, index * 300);
    });
  }, []);
  
  return (
    <FeatureContainer>
      <FeatureHeader>
        <FeatureTitle>Upcoming Events</FeatureTitle>
      </FeatureHeader>
      
      <FeatureContent>
        <EventList>
          {visibleEvents.map((event, index) => (
            <EventItem key={event.id} index={index}>
              <EventDate>
                <span className="month">{event.date.month}</span>
                <span className="day">{event.date.day}</span>
                <span className="year">{event.date.year}</span>
              </EventDate>
              <EventDetails>
                <EventTitle>{event.title}</EventTitle>
                <EventDescription>{event.description}</EventDescription>
                <div>
                  {event.tags.map(tag => (
                    <EventTag key={tag}>{tag}</EventTag>
                  ))}
                </div>
              </EventDetails>
            </EventItem>
          ))}
        </EventList>
      </FeatureContent>
    </FeatureContainer>
  );
};

export default FeatureSection; 