import React, { useState, useEffect } from 'react';
import styled, { keyframes } from 'styled-components';

const fadeIn = keyframes`
  from {
    opacity: 0;
    transform: scale(0.95);
  }
  to {
    opacity: 1;
    transform: scale(1);
  }
`;

const pulse = keyframes`
  0% {
    transform: scale(1);
  }
  50% {
    transform: scale(1.02);
  }
  100% {
    transform: scale(1);
  }
`;

const GalleryContainer = styled.div`
  width: 100%;
  max-width: 1000px;
  margin: 40px auto;
  padding: 20px;
  position: relative;
  z-index: 2;
  background: linear-gradient(to bottom, rgba(58, 43, 131, 0.7), rgba(42, 27, 99, 0.8));
  border-radius: 15px;
  box-shadow: 0 10px 30px rgba(153, 102, 255, 0.3);
`;

const GalleryTitle = styled.h2`
  color: white;
  font-size: 2.5rem;
  margin-bottom: 20px;
  text-align: center;
  text-transform: uppercase;
  text-shadow: 0 0 10px var(--neon-red), 0 0 20px var(--neon-red);
  animation: ${pulse} 2s infinite;
  
  @media (max-width: 768px) {
    font-size: 1.8rem;
  }
`;

const GalleryGrid = styled.div`
  display: grid;
  grid-template-columns: repeat(2, 1fr);
  gap: 20px;
  
  @media (max-width: 768px) {
    grid-template-columns: 1fr;
  }
`;

const HighlightCard = styled.div`
  position: relative;
  border-radius: 10px;
  overflow: hidden;
  box-shadow: 0 5px 15px rgba(58, 43, 131, 0.6);
  animation: ${fadeIn} 0.5s ease-out;
  animation-fill-mode: both;
  animation-delay: ${props => props.index * 0.2}s;
  transform-origin: center;
  transition: transform 0.3s ease, box-shadow 0.3s ease;
  height: 300px;
  
  &:hover {
    transform: translateY(-5px) scale(1.02);
    box-shadow: 0 10px 25px rgba(255, 51, 102, 0.5);
  }
  
  @media (max-width: 768px) {
    height: 250px;
  }
`;

const HighlightImage = styled.img`
  width: 100%;
  height: 100%;
  object-fit: cover;
  transition: transform 0.5s ease;
  
  ${HighlightCard}:hover & {
    transform: scale(1.05);
  }
`;

const HighlightOverlay = styled.div`
  position: absolute;
  bottom: 0;
  left: 0;
  right: 0;
  background: linear-gradient(to top, rgba(42, 27, 99, 0.9), transparent);
  padding: 20px;
  color: white;
  transform: translateY(70px);
  transition: transform 0.3s ease;
  
  ${HighlightCard}:hover & {
    transform: translateY(0);
  }
`;

const HighlightTitle = styled.h3`
  font-size: 1.5rem;
  margin-bottom: 5px;
  font-weight: 700;
`;

const HighlightDescription = styled.p`
  font-size: 0.9rem;
  margin-top: 10px;
  opacity: 0;
  transition: opacity 0.3s ease, transform 0.3s ease;
  transform: translateY(10px);
  
  ${HighlightCard}:hover & {
    opacity: 1;
    transform: translateY(0);
  }
`;

const TextScroller = styled.div`
  overflow: hidden;
  width: 100%;
  margin: 30px 0;
  background: linear-gradient(to right, rgba(255, 51, 102, 0.2), rgba(255, 51, 102, 0.4), rgba(255, 51, 102, 0.2));
  padding: 15px 0;
  border-radius: 5px;
  border: 1px solid rgba(255, 51, 102, 0.3);
`;

const ScrollingText = styled.div`
  white-space: nowrap;
  animation: scroll 20s linear infinite;
  color: white;
  font-size: 1.5rem;
  font-weight: 700;
  text-shadow: 0 0 5px rgba(255, 204, 0, 0.7);
  
  @keyframes scroll {
    0% {
      transform: translateX(100%);
    }
    100% {
      transform: translateX(-100%);
    }
  }
`;

const highlights = [
  {
    id: 1,
    image: '/mrbp.png',
    title: 'Extreme Challenges',
    description: 'Join MrBeast for mind-blowing challenges with incredible prizes!'
  },
  {
    id: 2,
    image: '/mrbp1.png',
    title: 'Squid Game Competition',
    description: 'Hundreds of contestants competing for the ultimate prize money.'
  },
  {
    id: 3,
    image: '/mrbp2.png',
    title: 'Team Challenges',
    description: 'Work together in intense team-based competitions!'
  },
  {
    id: 4,
    image: '/mrbp3.png',
    title: 'Massive Prize Pool',
    description: 'Compete for the chance to win life-changing amounts of money!'
  }
];

const HighlightGallery = () => {
  const [visibleHighlights, setVisibleHighlights] = useState([]);
  
  useEffect(() => {
    // Animate highlights appearing one by one
    const timer = setTimeout(() => {
      setVisibleHighlights(highlights);
    }, 500);
    
    return () => clearTimeout(timer);
  }, []);
  
  return (
    <GalleryContainer>
      <GalleryTitle>Mr Beast Highlights</GalleryTitle>
      
      <TextScroller>
        <ScrollingText>
          REGISTRATION OPEN NOW • OVER $6,000,000 IN PRIZES • SEASON 2 COMING SOON • LAST SEASON'S WINNER TOOK HOME $456,000 • DON'T MISS YOUR CHANCE
        </ScrollingText>
      </TextScroller>
      
      <GalleryGrid>
        {visibleHighlights.map((highlight, index) => (
          <HighlightCard key={highlight.id} index={index}>
            <HighlightImage src={highlight.image} alt={highlight.title} />
            <HighlightOverlay>
              <HighlightTitle>{highlight.title}</HighlightTitle>
              <HighlightDescription>{highlight.description}</HighlightDescription>
            </HighlightOverlay>
          </HighlightCard>
        ))}
      </GalleryGrid>
    </GalleryContainer>
  );
};

export default HighlightGallery; 